@@ -34,13 +34,23 @@
 
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
+#if defined(__OpenBSD__)
+# include <sys/dvdio.h>
+#elif defined(__linux__)  /* FIXME */
+# include <linux/cdrom.h>
+#else
+# error "FIX include for dvd ioctls"
+#endif
+
 /* Minimum length a title needs to be
  *  to be taken into consideration
  *  (= be shown in the list) in seconds */
@@ -49,6 +59,9 @@
 /* 8 audio streams is the maximum */
 #define MAX_AUDIO_STREAMS 8
 
+/* 32 subpicture streams is the maximum */
+#define MAX_SUBPICTURE_STREAMS 32
+
 /* how often we poll for a media 
  * change, if we have to do it ourselves */
 #define MEDIA_POLL_FREQ (1*1000)
@@ -83,13 +96,30 @@ enum
 
 enum
 {
-	AUDIO_TYPE_UNDEFINED,
+	AUDIO_TYPE_UNDEFINED = 0,
 	AUDIO_TYPE_NORMAL,
 	AUDIO_TYPE_IMPAIRED,
 	AUDIO_TYPE_COMMENTS1,
 	AUDIO_TYPE_COMMENTS2  /* music score? */
 };
 
+/* FIXME: are these accurate? (and for real?) */
+enum
+{
+	SUBPICTURE_TYPE_UNDEFINED = 0,
+	SUBPICTURE_TYPE_NORMAL = 1,
+	SUBPICTURE_TYPE_LARGE = 2,
+	SUBPICTURE_TYPE_CHILDREN = 3,
+	SUBPICTURE_TYPE_NORMAL_CAPTIONS = 5,
+	SUBPICTURE_TYPE_LARGE_CAPTIONS = 6,
+	SUBPICTURE_TYPE_CHILDRENS_CAPTIONS = 7,
+	SUBPICTURE_TYPE_FORCED = 9,
+	SUBPICTURE_TYPE_DIRECTORS_COMMENTS = 13,
+	SUBPICTURE_TYPE_DIRECTORS_COMMENTS_LARGE = 14,
+	SUBPICTURE_TYPE_DIRECTORS_COMMENTS_FOR_CHILDREN = 15
+};
+
+
 struct _ThDiscDrivePrivate
 {
 	gchar         *udi;         /* HAL device ID string         */
@@ -125,20 +155,27 @@ struct _ThTitleInfo
 
 	guint num_chapters;
 	
-	guint video_aspect;
+	guint frame_rate;
+	guint video_aspect;      /* 0 = 4:3, otherwise 16:9 */ 
+
 	guint size_x, size_y;
 
 	gchar title_id[64];      /* disc ID + title number as string      */
 	gchar volume_id[128];    /* title of disc                         */
 	guint num_audio_streams; /* number of (interesting) audio streams */
-	struct 
-	{
+	struct {
 		guint         aid;      /* this is the _logical_ audio stream number */
 		guint         phys_aid; /* this is the physical stream number on which
 		                         * the mpeg demuxer will base the pad name   */
 		audio_attr_t  attr;
-	} 
-	audio_streams[MAX_AUDIO_STREAMS];
+	} audio_streams[MAX_AUDIO_STREAMS];
+
+	guint num_subpicture_streams; /* number of interesting subpicture streams */
+	struct {
+		guint         sid;
+		guint         phys_sid;
+		subp_attr_t   attr;
+	} subpicture_streams[MAX_SUBPICTURE_STREAMS];
 };
 
 static void             disc_drive_class_init       (ThDiscDriveClass *klass);
@@ -498,6 +535,26 @@ dd_audio_stream_get_format_str (audio_attr_t *attr)
 	return _(formats[attr->audio_format]);
 }
 
+static const gchar *
+dd_audio_stream_get_decoder (audio_attr_t *attr)
+{
+  switch (attr->audio_format) {
+    case AUDIO_FORMAT_AC3:
+      return g_intern_static_string ("a52dec");
+    case AUDIO_FORMAT_MPEG1:
+      return g_intern_static_string ("mad");
+    case AUDIO_FORMAT_LPCM:
+      return g_intern_static_string ("dvdlpcmdec");
+    case AUDIO_FORMAT_DTS:
+      return g_intern_static_string ("dtsdec");
+    case AUDIO_FORMAT_MPEG2EXT:
+    case AUDIO_FORMAT_UNKNOWN:
+    case AUDIO_FORMAT_SDDS:
+      break;
+  }
+  return NULL;
+}
+
 /***************************************************************************
  *
  *   dd_audio_stream_get_channel_str
@@ -538,6 +595,28 @@ dd_audio_stream_get_language_str_short (audio_attr_t *attr)
 	return lang;
 }
 
+/***************************************************************************
+ *
+ *   dd_subp_stream_get_language_str_short
+ *
+ ***************************************************************************/
+
+static const gchar *
+dd_subp_stream_get_language_str_short (subp_attr_t *attr)
+{
+	static gchar lang[3];
+	
+	lang[0] = (attr->lang_code >> 8);
+	lang[1] = (attr->lang_code & 0xFF);
+	lang[2] = 0x00;
+
+	if (!g_ascii_isalnum (lang[0])
+	 || !g_ascii_isalnum (lang[1]))
+		lang[0] = '\0';
+
+	return lang;
+}
+
 /***************************************************************************
  *
  *   dd_dump_title_info
@@ -547,21 +626,29 @@ dd_audio_stream_get_language_str_short (audio_attr_t *attr)
 static void
 dd_dump_title_info (ThTitleInfo *info)
 {
+	const gchar *rates[5] = { "? (0)", "25.00", "? (2)", "29.97", "???" };
 	gint j;
 
-	th_log (" [%02u] [%02u:%02u:%02u] [Video 00] [%2u %s]\t[%s]\t[%ux%u]\n", info->num+1, 
-	         info->hours, info->mins, info->secs,
+	th_log (" [%02u] [%02u:%02u:%02u] [Video 00] [%2u %s]\t[%4s] [%ux%u] [%s fps]\n",
+	         info->num+1, info->hours, info->mins, info->secs,
 	         info->num_chapters, (info->num_chapters == 1) ? _("chapter") : _("chapters"),
 	         (info->video_aspect == 0) ? "4:3" : "16:9",
-	         info->size_x, info->size_y);
+	         info->size_x, info->size_y,
+	         rates[CLAMP (info->frame_rate, 0, 4)]);
 
 	for (j = 0;  j < info->num_audio_streams;  ++j)
 	{
-		th_log ("      [Audio %02u] [%s] %s, %s\n", info->audio_streams[j].phys_aid,
+		th_log ("      [Audio %02u] [%s] %s, %s\n",info->audio_streams[j].phys_aid,
 		         dd_audio_stream_get_language_str_short (&info->audio_streams[j].attr),
 		         dd_audio_stream_get_format_str (&info->audio_streams[j].attr),
 		         dd_audio_stream_get_channel_str (&info->audio_streams[j].attr));
 	}
+
+	for (j = 0;  j < info->num_subpicture_streams;  ++j)
+	{
+		th_log ("      [Sub   %02u] [%s]\n", info->subpicture_streams[j].phys_sid,
+		         dd_subp_stream_get_language_str_short (&info->subpicture_streams[j].attr));
+	}
 }
 
 
@@ -579,7 +666,7 @@ dd_job_add_audio_streams (ThJob *job, ThTitleInfo *info)
 	for (i = 0;  i < info->num_audio_streams;  ++i)
 	{
 		audio_attr_t *attr;
-		const gchar  *lang_str, *chan_str, *fmt_str;
+		const gchar  *lang_str, *chan_str, *fmt_str, *decoder;
 		gchar        *desc, *pad_name;
 		
 		attr = &info->audio_streams[i].attr;
@@ -587,16 +674,47 @@ dd_job_add_audio_streams (ThJob *job, ThTitleInfo *info)
 		fmt_str  = dd_audio_stream_get_format_str (attr); 
 		chan_str = dd_audio_stream_get_channel_str (attr);
 		lang_str = dd_audio_stream_get_language_str_short (attr);
-		
+		decoder = dd_audio_stream_get_decoder (attr);
+
 		desc = g_strdup_printf ("(%s, %s)", fmt_str, chan_str);
 		pad_name = g_strdup_printf ("audio_%02u", info->audio_streams[i].phys_aid);
 		th_job_add_audio_stream (job, info->audio_streams[i].aid,
-		                         pad_name, lang_str, desc);
+		                         pad_name, lang_str, desc, decoder);
 		g_free (pad_name);
 		g_free (desc);
 	}
 }
 
+/***************************************************************************
+ *
+ *   dd_job_add_subpicture_streams
+ *
+ ***************************************************************************/
+
+static void
+dd_job_add_subpicture_streams (ThJob *job, ThTitleInfo *info)
+{
+	gint i;
+
+	for (i = 0;  i < info->num_subpicture_streams;  ++i)
+	{
+		subp_attr_t *attr;
+		const gchar *lang_str;
+		gchar       *pad_name;
+		
+		attr = &info->subpicture_streams[i].attr;
+		
+		lang_str = dd_subp_stream_get_language_str_short (attr);
+		
+		pad_name = g_strdup_printf ("subpicture_%02u",
+		    info->subpicture_streams[i].phys_sid);
+		th_job_add_subpicture_stream (job, info->subpicture_streams[i].sid,
+		                              pad_name, lang_str);
+		g_free (pad_name);
+	}
+}
+
+
 /***************************************************************************
  *
  *   disk_drive_import_titles_from_pipe_data
@@ -640,7 +758,29 @@ disk_drive_import_titles_from_pipe_data (ThDiscDrive *drive, ThTitleInfo *titles
 			                  i, secs, titles[i].num_chapters,
 			                  titles[i].size_x, titles[i].size_y);
 
+			if (titles[i].frame_rate == 3) {
+			  job->video_info.fps_num = 30000; /* 29.97 fps */
+			  job->video_info.fps_denom = 1001;
+			} else {
+			  job->video_info.fps_num = 25;    /* 25.00 fps */
+			  job->video_info.fps_denom = 1;
+			}
+
+			if (titles[i].video_aspect == 0) {
+			  job->video_info.aspect_num = 4;
+			  job->video_info.aspect_denom = 3;
+			} else {
+			  job->video_info.aspect_num = 16;
+			  job->video_info.aspect_denom = 9;
+			}
+			job->video_info.aspect_num *= titles[i].size_y;
+			job->video_info.aspect_denom *= titles[i].size_x;
+
+			th_simplify_fraction (&job->video_info.aspect_num,
+			    &job->video_info.aspect_denom);
+
 			dd_job_add_audio_streams (job, &titles[i]);
+			dd_job_add_subpicture_streams (job, &titles[i]);
 
 			drive->priv->titles = g_list_append (drive->priv->titles, job);
 		}
@@ -729,7 +869,8 @@ dd_ioc_read_cb (GIOChannel *ioc, GIOCondition cond, ThDiscDrive *drive)
 	
 	if ((cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL)))
 	{
-		th_log ("pipe hangup. %u bytes in total\n", drive->priv->ioc_buf->len);
+		th_log ("pipe hangup. %" G_GSIZE_FORMAT " bytes in total\n",
+                        drive->priv->ioc_buf->len);
 		
 		g_assert (drive->priv->titles == NULL);
 		
@@ -749,8 +890,11 @@ dd_ioc_read_cb (GIOChannel *ioc, GIOCondition cond, ThDiscDrive *drive)
 		}
 		else
 		{
-			g_warning ("%s: unexpected number of bytes received (%u is not a multiple of %u)\n", 
-			           G_STRLOC, drive->priv->ioc_buf->len, sizeof (ThTitleInfo));
+			g_warning ("%s: unexpected number of bytes received "
+                                   "(%" G_GSIZE_FORMAT "u is not a multiple "
+                                   "of %" G_GSIZE_FORMAT ")\n", G_STRLOC, 
+			           drive->priv->ioc_buf->len,
+			           sizeof (ThTitleInfo));
 		}
 		
 		drive->priv->ioc = NULL;
@@ -930,7 +1074,8 @@ disc_drive_get_title_set_audio_info (ThDiscDrive  *drive,
 
 		attr = vts_file->vtsi_mat->vts_audio_attr[i];
 
-		th_log ("%3u  audio_%02u (%02u)  %s  %s, %-8s  (type = %u, ext = %u)\n", 
+		th_log ("%3u  audio_%02u      (%02u)  %s  %s, %-8s  "
+		        "(type = %u, ext = %u)\n", 
 		         tinfo->num + 1, phys_streamid, i, 
 			       dd_audio_stream_get_language_str_short (&attr),
 			       dd_audio_stream_get_format_str (&attr),
@@ -978,6 +1123,88 @@ disc_drive_get_title_set_audio_info (ThDiscDrive  *drive,
 	}
 }
 
+/***************************************************************************
+ *
+ *   disc_drive_get_title_set_subp_info
+ *
+ *   Extracts the subtitle streams which interest us
+ *
+ ***************************************************************************/
+
+static void
+disc_drive_get_title_set_subp_info (ThDiscDrive  *drive, 
+                                    ifo_handle_t *vts_file, 
+                                    pgc_t        *pgc,
+                                    ThTitleInfo  *tinfo)
+{
+	guint i;
+
+	tinfo->num_subpicture_streams = 0;
+	
+	g_return_if_fail (pgc != NULL);
+
+	for (i = 0;  i < vts_file->vtsi_mat->nr_of_vts_subp_streams;  ++i)
+	{
+		subp_attr_t  attr;
+		guint phys_streamid;
+		gint j;
+
+		/* subpicture stream not present? */ 
+		if ((pgc->subp_control[i] & 0x80000000) == 0)
+			continue;
+
+		/* get stream ID (depending on aspect ratio) (FIXME?) */
+		if (tinfo->video_aspect == 0) {
+			/* 4:3 */
+			phys_streamid = (pgc->subp_control[i] >> 16) & 0x3;
+		} else {
+			/* 16:9 */
+			phys_streamid = (pgc->subp_control[i] >> 16) & 0x3;
+		}
+
+		attr = vts_file->vtsi_mat->vts_subp_attr[i];
+
+		th_log ("%3u  subpicture_%02u (%02u)  %s                 "
+		        "(type = %u, ext = %u, coding = %u)\n", 
+		         tinfo->num + 1, phys_streamid, i, 
+		         dd_subp_stream_get_language_str_short (&attr),
+		         attr.type, attr.lang_extension, attr.code_mode);
+
+		if (attr.lang_code == 0xffff)
+			attr.lang_code = 0;
+
+		/* ignore subpicture streams without specified language */
+		if (attr.type != 1 || attr.lang_code == 0)
+			continue;
+
+		/* ignore subpicture streams that are not run-length encoded */
+		if (attr.code_mode != 0)
+			continue;
+
+		/* FIXME: allow these other types too? */
+		if (attr.lang_extension >= SUBPICTURE_TYPE_LARGE)
+			continue;
+
+		/* Check if we already have a stream in the same language */
+		for (j = 0;  j < tinfo->num_subpicture_streams;  ++j) {
+			if (tinfo->subpicture_streams[j].attr.lang_code == attr.lang_code)
+				break; /* => this stream will not be added below */
+		}
+			
+		/* Don't have a stream in this language yet? add it */
+		if (j == tinfo->num_subpicture_streams) {
+			tinfo->subpicture_streams[tinfo->num_subpicture_streams].sid  = i;
+			tinfo->subpicture_streams[tinfo->num_subpicture_streams].phys_sid  = phys_streamid;
+			tinfo->subpicture_streams[tinfo->num_subpicture_streams].attr = attr;
+				
+			++tinfo->num_subpicture_streams;
+				
+			if (tinfo->num_subpicture_streams == MAX_AUDIO_STREAMS)
+				return; /* bail out if more streams than expected/allowed */
+		}
+	}
+}
+
 /***************************************************************************
  *
  *   disc_drive_get_title_set_info
@@ -1027,6 +1254,7 @@ disc_drive_get_title_set_info (ThDiscDrive  *drive,
 	tinfo->num_chapters = tt_srpt->title[tinfo->num].nr_of_ptts;
 	
 	/* video stream info */
+	tinfo->frame_rate = (time->frame_u >> 6) & 0x03;
 	tinfo->video_aspect = vts_file->vtsi_mat->vts_video_attr.display_aspect_ratio;
 	
 	if (vts_file->vtsi_mat->vts_video_attr.picture_size == 0)
@@ -1046,12 +1274,94 @@ disc_drive_get_title_set_info (ThDiscDrive  *drive,
 		tinfo->size_y = 576;
 	
 	disc_drive_get_title_set_audio_info (drive, vts_file, cur_pgc, tinfo);
+	disc_drive_get_title_set_subp_info (drive, vts_file, cur_pgc, tinfo);
 
 	ifoClose (vts_file);
 	
 	return TRUE;
 }
 
+/***************************************************************************
+ *
+ *   disc_drive_query_rpc_status
+ *
+ *   Query the region code settings of the drive (for RPC-II drives), so
+ *   we can show an appropriate error message if there's a problem
+ *
+ ***************************************************************************/
+
+static void
+disc_drive_query_rpc_status (ThDiscDrive  *drive)
+{
+#if defined (DVD_AUTH) && defined (DVD_LU_SEND_RPC_STATE)
+  dvd_authinfo auth_info;
+  int fd, ret, i;
+
+  fd = open (drive->priv->device, O_RDONLY | O_NONBLOCK);
+  if (fd < 0) {
+    th_log ("Can't open dvd device: %s\n", g_strerror (errno));
+    return;
+  }
+
+  memset (&auth_info, 0, sizeof (auth_info));
+  auth_info.type = DVD_LU_SEND_RPC_STATE;
+
+  ret = ioctl (fd, DVD_AUTH, &auth_info);
+
+  if (ret < 0) {
+    th_log ("ioctl DVD_AUTH failed: %s\n", g_strerror (errno));
+    goto out;
+  }
+   
+  th_log ("Current Region Code settings:\n");
+  th_log ("  RPC Scheme                           : %s\n",
+      (auth_info.lrpcs.rpc_scheme == 1) ? "RPC-II" : "Unknown");
+  switch (auth_info.lrpcs.type) {
+    case 0:
+      th_log ("  Type                                 : NO REGION CODE SET YET\n");
+      break;
+    case 1:
+      th_log ("  Type                                 : REGION CODE SET\n");
+      break;
+    case 2:
+      th_log ("  Type                                 : LAST CHANCE\n");
+      break;
+    case 3:
+      th_log ("  Type                                 : REGION CODE PERMANENT\n");
+      break;
+    default:
+      break;
+  }
+  th_log ("  Vendor resets left                   : %d\n", auth_info.lrpcs.vra);
+  th_log ("  User controlled changes left         : %d\n", auth_info.lrpcs.ucca);
+
+  th_log ("  Region_mask=0x%02X\n", auth_info.lrpcs.region_mask);
+  if (auth_info.lrpcs.region_mask == 0xff) {
+    th_log ("  Drive will play discs from region(s) : %s\n",
+        "NONE, you must set a region first");
+  } else {
+    gchar regions[256] = { 0, };
+
+    for (i = 0; i < 8; ++i) {
+      if ((auth_info.lrpcs.region_mask & (1 << i)) == 0) {
+        regions[i * 2] = (i + 1) + '0';
+        regions[(i * 2)+1] = ' ';
+      }
+    }
+    th_log ("  Drive will play discs from region(s) : %s\n", regions);
+  }
+
+out:
+
+  close (fd);
+
+#else
+
+  th_log ("Can't query DVD RPC status on this operating system.\n");
+
+#endif
+}
+
 /***************************************************************************
  *
  *   disc_drive_get_title_info_real
@@ -1081,7 +1391,10 @@ disc_drive_get_title_info_real (ThDiscDrive *drive, gint fd)
 		g_warning ("DVDOpen (%s) failed.\n", drive->priv->device);
 		return;
 	}
-	
+
+	/* check region code settings, if possible */
+	disc_drive_query_rpc_status (drive);
+
 	if (DVDDiscID (dvd, discid) != 0)
 	{
 		g_warning ("DVDDiscID (%s) failed.\n", drive->priv->device);
@@ -1147,7 +1460,8 @@ disc_drive_get_title_info_real (ThDiscDrive *drive, gint fd)
 		{
 			ret = write (fd, buf, left);
 	
-			th_log ("%d = write (%d, %p, %u)\n", ret, fd, buf, left);
+			th_log ("%d = write (%d, %p, %u)\n",
+			        (int) ret, fd, buf, (guint) left);
 
 			if (ret < 0)
 			{
@@ -1162,7 +1476,8 @@ disc_drive_get_title_info_real (ThDiscDrive *drive, gint fd)
 			buf += ret;
 			left -= ret;
 		
-			th_log ("\t=>written = %u, buf = %p, left = %u\n", written, buf, left);
+			th_log ("\t=>written = %u, buf = %p, left = %u\n",
+			        (guint) written, buf, (guint) left);
 		}
 		while (ret > 0 && left > 0);
 	}
