@@ -56,6 +56,8 @@ struct _ThJobConfigDialogPrivate
 	
 	gulong        lang_combobox_sigid;
 	GtkWidget    *lang_combobox;
+	gulong        lang_all_sigid;
+	GtkWidget    *lang_all;
 	GtkListStore *lang_list;
 	
 	GtkWidget    *picsize_combobox;
@@ -89,6 +91,8 @@ static gboolean         job_config_dialog_configure_job    (ThJobConfigDialog *j
 
 static void             job_config_dialog_update_crop_labels (ThJobConfigDialog *jcd);
 
+static void             job_config_dialog_update_file_size_range (ThJobConfigDialog *jcd);
+
 static void             job_config_dialog_job_update_picsizes (ThJobConfigDialog *jcd);
 
 static void             job_config_dialog_fill_language_combobox (ThJobConfigDialog *jcd);
@@ -456,6 +460,35 @@ job_config_dialog_language_changed (ThJobConfigDialog *jcd, GtkComboBox *cbox)
 	g_object_set (jcd->priv->job, "audio-stream-id", aid, NULL);
 }
 
+/***************************************************************************
+ *
+ *   job_config_dialog_language_all_toggled
+ *
+ ***************************************************************************/
+
+static void
+job_config_dialog_language_all_toggled (ThJobConfigDialog *jcd, GtkCheckButton *cbox)
+{
+  gboolean active;
+
+  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cbox));
+
+  /* Disable combobox, set all audio flag */
+  gtk_widget_set_sensitive (jcd->priv->lang_combobox, !active);
+  g_object_set (jcd->priv->job, "audio-all", active, NULL);
+
+  job_config_dialog_update_file_size_range (jcd);
+
+  /* force re-calculation of effective video bitrate if the number of audio
+   * streams has changed */
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (jcd->priv->filesize_radiobutton))) {
+    gdouble target_size;
+
+    g_object_get (jcd->priv->job, "target-output-size", &target_size, NULL);
+    g_object_set (jcd->priv->job, "target-output-size", target_size, NULL);
+  }
+}
+
 /***************************************************************************
  *
  *   job_config_dialog_setup_language_combobox
@@ -484,6 +517,10 @@ job_config_dialog_setup_language_combobox (ThJobConfigDialog *jcd)
 		g_signal_connect_swapped (jcd->priv->lang_combobox, "changed",
 		                          G_CALLBACK (job_config_dialog_language_changed),
 		                          jcd);
+	jcd->priv->lang_all_sigid = 
+		g_signal_connect_swapped (jcd->priv->lang_all, "toggled",
+		                          G_CALLBACK (job_config_dialog_language_all_toggled),
+		                          jcd);
 }
 
 /***************************************************************************
@@ -510,10 +547,11 @@ job_config_dialog_fill_language_combobox (ThJobConfigDialog *jcd)
 
 	g_object_get (jcd->priv->job, 
 	              "audio-stream-id", &default_aid,
-	              "num-audio-streams", &num_audio_streams, 
 	              NULL);
 
 	th_log ("default_aid = %u\n", default_aid);
+
+	num_audio_streams = jcd->priv->job->audio_streams->len;
 	for (i = 0;  i < num_audio_streams;  ++i)
 	{
 		const gchar *lang, *desc, *lang_name;
@@ -681,6 +719,7 @@ th_job_config_dialog_new (void)
 	                                 "th-filename-label",      &jcd->priv->fn_label,
 	                                 "th-filename-button",     &jcd->priv->fn_button,
 	                                 "th-language-combobox",   &jcd->priv->lang_combobox,
+	                                 "th-language-all",        &jcd->priv->lang_all,
 	                                 "th-picsize-combobox",    &jcd->priv->picsize_combobox,
 	                                 "th-crop-top-label",      &jcd->priv->crop_top_label,
 	                                 "th-crop-left-label",     &jcd->priv->crop_left_label,
@@ -800,6 +839,23 @@ th_job_config_dialog_configure_jobs (ThJobConfigDialog *jcd, GList *jobs)
 	return TRUE;
 }
 
+static void
+job_config_dialog_update_file_size_range (ThJobConfigDialog *jcd)
+{
+  GtkSpinButton *filesize_spin;
+  gdouble  min_fsize, max_fsize;
+
+  filesize_spin = GTK_SPIN_BUTTON (jcd->priv->filesize_spinbutton);
+
+  th_job_get_file_size_range (jcd->priv->job, &min_fsize, &max_fsize);
+
+  /* +/- 0.5 because of double => int conversion (don't 
+   *   want to end up with a value too low or high */
+  g_signal_handler_block (filesize_spin, jcd->priv->filesize_spinbutton_sigid);
+  gtk_spin_button_set_range (filesize_spin, min_fsize + 0.5, max_fsize - 0.5);
+  g_signal_handler_unblock (filesize_spin, jcd->priv->filesize_spinbutton_sigid);
+}
+
 /***************************************************************************
  *
  *   job_config_dialog_configure_job
@@ -812,7 +868,7 @@ job_config_dialog_configure_job (ThJobConfigDialog *jcd, ThJob *job)
 	ThPictureSize picsize;
 	GtkTreeIter   iter;
 	gboolean      ret = TRUE;
-	gdouble       min_fsize, max_fsize, target_output_size;
+	gdouble       target_output_size;
 	gulong        notify_id1;
 	guint         title_num, target_quality, response_id;
 	gchar        *title_tag, *comment_tag;
@@ -823,17 +879,7 @@ job_config_dialog_configure_job (ThJobConfigDialog *jcd, ThJob *job)
 	jcd->priv->job = job;
 
 	job_config_dialog_update_crop_labels (jcd);
-
-	th_job_get_file_size_range (job, &min_fsize, &max_fsize);
-	
-	/* +/- 0.5 because of double => int conversion (don't 
-	 *   want to end up with a value too low or high */
-	g_signal_handler_block (jcd->priv->filesize_spinbutton, jcd->priv->filesize_spinbutton_sigid);
-	
-	gtk_spin_button_set_range (GTK_SPIN_BUTTON (jcd->priv->filesize_spinbutton),
-	                           min_fsize + 0.5, max_fsize - 0.5);
-	
-	g_signal_handler_unblock (jcd->priv->filesize_spinbutton, jcd->priv->filesize_spinbutton_sigid);
+	job_config_dialog_update_file_size_range (jcd);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (jcd->priv->quality_radiobutton), TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (jcd->priv->filesize_radiobutton), TRUE);
@@ -870,6 +916,12 @@ job_config_dialog_configure_job (ThJobConfigDialog *jcd, ThJob *job)
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (jcd->priv->quality_spinbutton), target_quality);
 	}
 
+	/* no need to show the 'all languages' checkbox if there is only one */
+	if (job->audio_streams->len == 1) {
+	  gtk_widget_hide (jcd->priv->lang_all);
+	} else {
+	  gtk_widget_show (jcd->priv->lang_all);
+	}
 	
 	s = g_string_new (NULL);
 	g_string_append (s, "<b>");
