@@ -15,6 +15,10 @@
  *                                                                         *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "th-job-progress-dialog.h"
 #include "th-job.h"
 #include "th-power-manager-agent.h"
@@ -627,7 +631,19 @@ job_progress_calc_time_left (ThJobProgressDialog *jpd, guint pos_msecs)
 	/* note: secs_left might be negative, as length is not exact */
 	cstr = job_get_human_time_interval_str ((gint) secs_left);
 	eta = time (NULL) + (time_t) MAX (15.0, secs_left);
-	strftime (timestr, sizeof (timestr), "%R", localtime (&eta));
+
+#ifdef HAVE_LOCALTIME_R
+	{
+	  struct tm tm;
+
+	  strftime (timestr, sizeof (timestr), "%R", localtime_r (&eta, &tm));
+	}
+#else
+	{
+	  strftime (timestr, sizeof (timestr), "%R", localtime (&eta));
+	}
+#endif
+
 	th_label_set_text (jpd->priv->timeleft_label, "%s [%sh]", cstr, timestr); 
 }
 
