@@ -1000,7 +1000,11 @@ th_job_save_config (ThJob *job)
 
 			if (valstr)
 			{
-				g_string_append_printf (xml, "  <%s>%s</%s>\n", props[n], valstr, props[n]);
+				gchar *val_escaped;
+
+				val_escaped = g_markup_escape_text (valstr, -1);
+				g_string_append_printf (xml, "  <%s>%s</%s>\n", props[n], val_escaped, props[n]);
+				g_free (val_escaped);
 				g_free (valstr);
 				valstr = NULL;
 			}
@@ -2193,7 +2197,7 @@ job_get_snapshot_from_current_buf (ThJob *j)
 	 *   chance to ref it (is this necessary?) */
 	do
 	{
-		curbuf = g_atomic_pointer_get (job_current_buf_addr);
+		curbuf = g_atomic_pointer_get ((gpointer *) job_current_buf_addr);
 	}
 	while (!g_atomic_pointer_compare_and_exchange ((gpointer *) job_current_buf_addr, curbuf, NULL));
 
