@@ -37,16 +37,33 @@ typedef enum
 	TH_PICTURE_SIZE_FULL          /* 1/1, 1.000, e.g. 768 x 576  */
 } ThPictureSize;
 
-
 typedef struct _ThJob        ThJob;
 typedef struct _ThJobClass   ThJobClass;
 typedef struct _ThJobPrivate ThJobPrivate;
 
+typedef struct {
+  guint    fps_num;        /* display frame rate */
+  guint    fps_denom;
+  guint    aspect_num;     /* video aspect */
+  guint    aspect_denom;
+} ThJobVideoInfo;
+
+typedef struct {
+  guint         aid;
+  const gchar  *lang;     /* interned string: ISO 639-1 language code or "\0" */
+  const gchar  *pad_name; /* interned string: demuxer pad name                */
+  const gchar  *desc;     /* interned string: description, e.g. format, chans */
+  const gchar  *decoder;  /* interned string: decoder element, e.g. "mad"     */
+} ThJobAudioStream;
+
 struct _ThJob
 {
-	GObject        object;
-	
-	ThJobPrivate  *priv;
+	GObject object;
+
+	ThJobVideoInfo         video_info;
+	GArray                *audio_streams;
+
+	ThJobPrivate          *priv;
 };
 
 struct _ThJobClass
@@ -83,10 +100,14 @@ void          th_job_get_file_size_range (ThJob *job, gdouble *p_min_size, gdoub
 
 void          th_job_set_filename_from_disc_title (ThJob *j, const gchar *disc_title);
 
-void          th_job_add_audio_stream (ThJob *j, guint aid, const gchar *pad_name, const gchar *lang, const gchar *description);
+void          th_job_add_audio_stream (ThJob *j, guint aid, const gchar *pad_name, const gchar *lang, const gchar *description, const gchar *decoder);
 
 gboolean      th_job_get_audio_stream (ThJob *j, guint n, guint *p_aid, const gchar **p_lang, const gchar **p_desc);
 
+void          th_job_add_subpicture_stream (ThJob *j, guint sid, const gchar *pad_name, const gchar *lang);
+
+gboolean      th_job_get_subpicture_stream (ThJob *j, guint n, guint *p_sid, const gchar **lang);
+
 gchar        *th_job_get_default_title (ThJob *j);
 
 #endif /* _th_job_h_included_ */
