@@ -25,7 +25,10 @@
 #include "gst-plugins/th-plugins.h"
 
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
+
 #include <gst/gst.h>
+#include <gst/pbutils/pbutils.h>
 
 #include <glib/gi18n.h>
 
@@ -251,6 +254,78 @@ print_system_info (void)
 	}
 }
 
+static void
+plugins_install_result_func (GstInstallPluginsReturn res, gpointer data)
+{
+  th_log ("Plugin install: %s\n", gst_install_plugins_return_get_name (res));
+
+  if (res == GST_INSTALL_PLUGINS_SUCCESS ||
+      res == GST_INSTALL_PLUGINS_PARTIAL_SUCCESS) {
+    th_log ("New plugins were installed, updating registry");
+    if (!gst_update_registry ()) {
+      th_log ("Registry update failed for some reason");
+    } else {
+      /* not sure what to do now; assuming the installer puts itself on top of
+       * the application window, the user couldn't have done much since startup.
+       * For ffdeinterlace we don't need to do much, it will be used when we
+       * start the encoding (if available). For new audio decoders we'd need to
+       * re-read the disc contents for them to be taken into account. */
+    }
+  }
+}
+
+static void
+install_missing_desirable_plugins (GtkWidget * appwindow)
+{
+  const gchar *desirable_features[] = { "dtsdec", "dvdlpcmdec",
+      "ffdeinterlace" };
+  GstInstallPluginsContext *ctx = NULL;
+  GPtrArray *missing = NULL;
+  guint i, xid;
+
+  /* must realize the applicatio window, so that widget->window is non-NULL */
+  gtk_widget_realize (appwindow);
+
+  ctx = gst_install_plugins_context_new ();
+  xid = GDK_WINDOW_XWINDOW (GTK_WIDGET (appwindow)->window);
+  gst_install_plugins_context_set_xid (ctx, xid);
+
+  missing = g_ptr_array_new ();
+  for (i = 0; i < G_N_ELEMENTS (desirable_features); ++i) {
+    const gchar *fname;
+
+    fname = desirable_features[i];
+    if (!gst_default_registry_check_feature_version (fname, 0, 10, 0)) {
+      GstElement *dummy;
+      GstMessage *msg;
+
+      th_log ("GStreamer element '%s' is desirable, but not installed\n", fname);
+      dummy = gst_pipeline_new (NULL);
+      /* FIXME: use gst_missing_element_installer_detail_new() once
+       * gst-plugins-base 0.10.15 is released and we can depend on it */
+      msg = gst_missing_element_message_new (dummy, fname);
+      g_ptr_array_add (missing,
+          gst_missing_plugin_message_get_installer_detail (msg));
+      gst_message_unref (msg);
+      gst_object_unref (dummy);
+    }
+  }
+  if (missing->len > 0) {
+    GstInstallPluginsReturn ret;
+
+    g_ptr_array_add (missing, NULL);
+
+    ret = gst_install_plugins_async ((gchar **) missing->pdata, ctx,
+        plugins_install_result_func, NULL);
+    if (ret != GST_INSTALL_PLUGINS_STARTED_OK)
+      th_log ("Plugin install: %s\n", gst_install_plugins_return_get_name (ret));
+
+    g_ptr_array_foreach (missing, (GFunc) g_free, NULL);
+  }
+  g_ptr_array_free (missing, TRUE);
+  gst_install_plugins_context_free (ctx);
+}
+
 /***************************************************************************
  *
  *   main
@@ -265,8 +340,11 @@ main (int argc, char **argv)
 		gpointer    appwin_ptr;
 	} run_info;
 
-	gtk_init (&argc, &argv);
+	/* must be called before any other GLib function */
+	g_thread_init (NULL);
+
 	gst_init (&argc, &argv);
+	gtk_init (&argc, &argv);
 
 	if (!check_required_plugins ())
 		g_error ("We are missing some required plugins. There's something wrong with the installation.\n");
@@ -293,6 +371,10 @@ main (int argc, char **argv)
 
 	g_object_add_weak_pointer (G_OBJECT (run_info.appwin), &run_info.appwin_ptr);
 
+	/* Try to have plugins that are desirable to have but are currently
+	 * missing installed automatically if the distro supports it */
+	install_missing_desirable_plugins (run_info.appwin);
+
 	th_app_window_run (TH_APP_WINDOW (run_info.appwin));
 
 	if (run_info.appwin != NULL)
