@@ -508,13 +508,13 @@ askfordevice:
 	
 	drive = th_disc_drive_new (dev, dev, _("Possible DVD Drive"), " ");
 
-	g_signal_emit_by_name (ts->priv->disc_pool, "drive-added", drive);
+	if (th_disc_drive_pool_add_drive (ts->priv->disc_pool, drive)) {
+	  g_object_set (drive, 
+	                "poll-media-required", TRUE, 
+	                "poll-media", TRUE, 
+	                NULL);
+	}
 
-	g_object_set (drive, 
-	              "poll-media-required", TRUE, 
-	              "poll-media", TRUE, 
-	              NULL);
-	
 	g_object_unref (drive); /* list store added ref */
 
 	title_selector_set_titles_list_msg (ts, MSG_DETECTING_DRIVES);
@@ -714,7 +714,7 @@ title_selector_popup_context_menu (ThTitleSelector *ts, GdkEventButton *event)
 
 	menu = gtk_menu_new ();
 	
-	menuitem = gtk_menu_item_new_with_label ("Preview");
+	menuitem = gtk_menu_item_new_with_label (_("Preview"));
 	
 	g_signal_connect_swapped (menuitem, "activate", 
 	                          G_CALLBACK (title_selector_title_preview),
@@ -1551,27 +1551,30 @@ th_title_selector_run (ThTitleSelector *ts)
 gboolean
 th_title_selector_add_vob_folder (ThTitleSelector *ts, const gchar *dir)
 {
-	ThDiscDrive *drive;
+  ThDiscDrive *drive;
 
-	g_return_val_if_fail (TH_IS_TITLE_SELECTOR (ts), FALSE);
-	g_return_val_if_fail (dir != NULL, FALSE);
-	g_return_val_if_fail (g_path_is_absolute (dir), FALSE);
+  g_return_val_if_fail (TH_IS_TITLE_SELECTOR (ts), FALSE);
+  g_return_val_if_fail (dir != NULL, FALSE);
+  g_return_val_if_fail (g_path_is_absolute (dir), FALSE);
 
-	/* FIXME: find nice string and make translatable */
-	drive = th_disc_drive_new (dir, dir, "Directory with .VOB Files", "");
+  drive = th_disc_drive_pool_get_drive_from_udi (ts->priv->disc_pool, dir);
 
-	g_return_val_if_fail (drive != NULL, FALSE);
+  if (drive == NULL) {
+    /* FIXME: find nice string and make translatable */
+    drive = th_disc_drive_new (dir, dir, "Directory with .VOB Files", "");
+    g_return_val_if_fail (drive != NULL, FALSE);
 
-	g_signal_emit_by_name (ts->priv->disc_pool, "drive-added", drive);
+    g_object_set (drive, "disc-title", dir, NULL);
 
-	g_object_set (drive, "disc-title", dir, NULL);
+    th_disc_drive_pool_add_drive (ts->priv->disc_pool, drive);
 
-	g_signal_emit_by_name (ts->priv->disc_pool, "disc-changed", drive);
+    /* the title selector list store holds a reference now */
+    g_object_unref (drive);
+  }
 
-	/* the title selector list store holds a reference now */
-	g_object_unref (drive);
+  g_signal_emit_by_name (ts->priv->disc_pool, "disc-changed", drive);
 
-	return TRUE;
+  return TRUE;
 }
 
 /***************************************************************************
@@ -1583,31 +1586,35 @@ th_title_selector_add_vob_folder (ThTitleSelector *ts, const gchar *dir)
 gboolean
 th_title_selector_add_iso_file (ThTitleSelector *ts, const gchar *iso_fn)
 {
-	ThDiscDrive *drive;
-	gchar *base_name;
+  ThDiscDrive *drive;
 
-	g_return_val_if_fail (TH_IS_TITLE_SELECTOR (ts), FALSE);
-	g_return_val_if_fail (iso_fn != NULL, FALSE);
-	g_return_val_if_fail (g_path_is_absolute (iso_fn), FALSE);
+  g_return_val_if_fail (TH_IS_TITLE_SELECTOR (ts), FALSE);
+  g_return_val_if_fail (iso_fn != NULL, FALSE);
+  g_return_val_if_fail (g_path_is_absolute (iso_fn), FALSE);
 
-	base_name = g_path_get_basename (iso_fn);
+  drive = th_disc_drive_pool_get_drive_from_udi (ts->priv->disc_pool, iso_fn);
 
-	/* FIXME: find nice string and make translatable */
-	drive = th_disc_drive_new (iso_fn, iso_fn, "ISO image", "");
+  if (drive == NULL) {
+    gchar *base_name;
 
-	g_return_val_if_fail (drive != NULL, FALSE);
+    base_name = g_path_get_basename (iso_fn);
 
-	g_signal_emit_by_name (ts->priv->disc_pool, "drive-added", drive);
+    /* FIXME: find nice string and make translatable */
+    drive = th_disc_drive_new (iso_fn, iso_fn, "ISO image", "");
 
-	g_object_set (drive, "disc-title", base_name, NULL);
+    g_return_val_if_fail (drive != NULL, FALSE);
 
-	g_signal_emit_by_name (ts->priv->disc_pool, "disc-changed", drive);
+    g_object_set (drive, "disc-title", base_name, NULL);
 
-	/* the title selector list store holds a reference now */
-	g_object_unref (drive);
+    th_disc_drive_pool_add_drive (ts->priv->disc_pool, drive);
 
-	g_free (base_name);
+    /* the title selector list store holds a reference now */
+    g_object_unref (drive);
+    g_free (base_name);
+  }
 
-	return TRUE;
+  g_signal_emit_by_name (ts->priv->disc_pool, "disc-changed", drive);
+
+  return TRUE;
 }
 
