@@ -594,6 +594,11 @@ th_mkdir_recursive (const gchar *basedir, const gchar *adddir, GError **err)
 static gchar *
 utils_find_mount_point (const gchar *fn)
 {
+#if defined(HAVE_MNTTAB) || defined(HAVE_MTAB)
+	FILE          *mount_table;
+#else
+# warning have neither mtab not mnttab - utils_find_mount_point() will be unimplemented
+#endif
 #ifdef HAVE_MTAB
 	struct mntent *mount_entry = NULL;
 #elif defined(HAVE_MNTTAB)
@@ -601,8 +606,6 @@ utils_find_mount_point (const gchar *fn)
 #endif
 	struct stat    s;
 	dev_t          mount_device;
-	FILE          *mount_table;
-
 	g_return_val_if_fail (fn != NULL, NULL);
 
 	if (stat (fn, &s) != 0)
